//===========================================================================
//
//  CRYSTALFONTZ CFAG320240CX family Demo Code
//    https://www.crystalfontz.com/product/cfag320240cxttit
//
//  The controller is a Epson S1D13700 or compatible:
//    https://www.crystalfontz.com/controllers/Epson/S1D13700/
//
//  Seeeduino v4.2, an open-source 3.3v capable Arduino clone.
//    https://www.seeedstudio.com/Seeeduino-V4.2-p-2517.html
//    https://github.com/SeeedDocument/SeeeduinoV4/raw/master/resources/Seeeduino_v4.2_sch.pdf
//===========================================================================
//
//  2019-08-19 Brent Crosby
//
//===========================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//============================================================================
// LCD control lines
// -------+-----+--------------------------+--------+
//   ARD  |Port | Functn                   | Wire   |
// -------+-----+--------------------------+--------+
//  #8/D8 | PB0 | Chip Enable Signal  (CS) | Purple |
//  #9/D9 | PB1 | Reset              (RES) | Blue   |
// #14/A0 | PC0 | Data/Instruction    (A0) | Yellow |
// #15/A1 | PC1 | Write               (WR) | Grey   |
// #16/A2 | PC2 | READ                (RD) | Brown  |
// #17/A3 | PC3 | Wait/Busy,inout \ (WAIT) | White  |
// -------+-----+--------------------------+--------+
//  #0/D0 | PD0 | LCD D10 (DB0)            | Black  |
//  #1/D1 | PD1 | LCD D11 (DB1)            | Brown  |
//  #2/D2 | PD2 | LCD D12 (DB2)            | Red    |
//  #3/D3 | PD3 | LCD D13 (DB3)            | Orange |
//  #4/D4 | PD4 | LCD D14 (DB4)            | Yellow |
//  #5/D5 | PD5 | LCD D15 (DB5)            | Green  |
//  #6/D6 | PD6 | LCD D16 (DB6) Share: Tx  | Blue   |
//  #7/D7 | PD7 | LCD D17 (DB7) Share: Rx* | Purple |
// -------+-----+--------------------------+--------+
// *You will need to disconnect D7 during programming.
// -----------+------+-----------------------------+------------
// The contrast and voltage generation is done on-board
// and pre-set from the factory. No external potentiometer
// is needed.
// -------------------------------------------------------------
#define CS_CLR_SELECT       (PORTB &= ~(0x01))
#define CS_SET_DESELECT     (PORTB |=  (0x01))
#define RESET_CLR_RESET     (PORTB &= ~(0x02))
#define RESET_SET_OPERATE   (PORTB |=  (0x02))

#define RS_CLR_DATA         (PORTC &= ~(0x01))
#define RS_SET_COMMAND      (PORTC |=  (0x01))
#define WR_CLR_WRITE        (PORTC &= ~(0x02))
#define WR_SET_IDLE         (PORTC |=  (0x02))
#define RD_CLR_READ         (PORTC &= ~(0x04))
#define RD_SET_IDLE         (PORTC |=  (0x04))

#define SET_LCD_PORT_OUT    (DDRD=0xFF)
#define SET_LCD_PORT_IN     (DDRD=0x00)
#define LCD_DATA_SET_OUT    PORTD
#define LCD_DATA_READ_IN    PIND
//============================================================================
// Depend on / enforce CS, RESET, WR, and RD idle / high / inactive.
//============================================================================
#include <Arduino.h>
#include "bitmaps.h"
//============================================================================
void write_data_S1D13700(uint8_t data)
  {
  //Apply the data to the port
  LCD_DATA_SET_OUT=data;
  //Select the data register
  RS_CLR_DATA;
  //Select the chip
  CS_CLR_SELECT;

  //Make the write pulse
  _delay_us(1); //emprical
  WR_CLR_WRITE;
  _delay_us(1); //emprical
  WR_SET_IDLE;
  _delay_us(1); //emprical
      
  //Deslect the chip
  CS_SET_DESELECT;
  }
//============================================================================
void write_command_S1D13700(uint8_t command)
  {
  //Apply the data to the port
  LCD_DATA_SET_OUT=command;
  //Select the command register
  RS_SET_COMMAND;
  //Select the chip
  CS_CLR_SELECT;

  //Make the write pulse
  _delay_us(1); //emprical
  WR_CLR_WRITE;
  _delay_us(1); //emprical
  WR_SET_IDLE;
  _delay_us(1); //emprical
        
  //Deslect the chip
  CS_SET_DESELECT;
  }
//============================================================================
const uint8_t Splash_Text_Screen[30][41] PROGMEM
  {"                                        ",  // 0
   "                                        ",  // 1
   "                                        ",  // 2
   "                                        ",  // 3
   "                                        ",  // 4
   "                                        ",  // 5
   "                                        ",  // 6
   "                                        ",  // 7
   "                                        ",  // 8
   "                                        ",  // 9
   "                                        ",  // 10
   "                                        ",  // 11
   "                                        ",  // 12
   "                                        ",  // 13
   "                                        ",  // 14
   "                                        ",  // 15
   "                                        ",  // 16
   "                                        ",  // 17
   "                                        ",  // 18
   "                                        ",  // 19
   "                               Built-in ",  // 20
   "                               5x7 font ",  // 21
   "                               on a     ",  // 22
   "                               separate ",  // 23
   "                               text     ",  // 24
   "                               layer.   ",  // 25
   "                               Can be   ",  // 26
   "                               combined ",  // 27
   "                               using AND",  // 28
   "                               OR or XOR"}; // 29  
//----------------------------------------------------------------------------
const uint8_t Text_Demo_Screen[30][41] PROGMEM =
  {"01234567890123456789012345678901 00 6789",  // 00
   "01234567890123456789012345678901 01 6789",  // 01
   "01234567890123456789012345678901 02 6789",  // 02
   "01234567890123456789012345678901 03 6789",  // 03
   "01234567890123456789012345678901 04 6789",  // 04
   "01234567890123456789012345678901 05 6789",  // 05
   "01234567890123456789012345678901 06 6789",  // 06
   "01234567890123456789012345678901 07 6789",  // 07
   "01234567890123456789012345678901 08 6789",  // 08
   "01234567890123456789012345678901 09 6789",  // 09
   "01234567890123456789012345678901 10 6789",  // 10
   "01234567890123456789012345678901 11 6789",  // 11
   "01234567890123456789012345678901 12 6789",  // 12
   "012345678                    901 13 6789",  // 13
   "012345678 40 col by 30 lines 901 14 6789",  // 14
   "012345678    of 5x7 text     901 15 6789",  // 15
   "012345678                    901 16 6789",  // 16
   "01234567890123456789012345678901 17 6789",  // 17
   "01234567890               678901 18 6789",  // 18
   "01234567890 ABCDEFGHIJKLM 678901 19 6789",  // 19
   "01234567890 NOPQRSTUVWXYZ 678901 20 6789",  // 20
   "01234567890 abcdefghijklm 678901 21 6789",  // 21
   "01234567890 nopqrstuvwxyz 678901 22 6789",  // 22
   "01234567890               678901 23 6789",  // 23
   "01234567890123456789012345678901 24 6789",  // 24
   "01234567890123456789012345678901 25 6789",  // 25
   "01234567890123456789012345678901 26 6789",  // 26
   "01234567890123456789012345678901 27 6789",  // 27
   "01234567890123456789012345678901 28 6789",  // 28
   "01234567890123456789012345678901 29 6789"}; // 29
//----------------------------------------------------------------------------
void Show_Text_Layer(const uint8_t *Text_Data)
  {
  uint8_t
    row;
  uint8_t
    col;
  uint16_t
    address;
  address=0;

  for(row=0;row<30;row++)
    {
    //Set the position, within the text layer
    write_command_S1D13700(0x46);
    write_data_S1D13700(address&0x00FF);
    write_data_S1D13700((address&0xFF00)>>8);
  
    // MEMWRITE command
    write_command_S1D13700(0x42);
    //Select the data register
    RS_CLR_DATA;
    //Select the chip
    CS_CLR_SELECT;

    for(col=0;col<40;col++)
      {
      //Get the data from flash, send it to the controller.
      LCD_DATA_SET_OUT=pgm_read_byte(Text_Data);
      //Make the write pulse
      _delay_us(1); //emprical
      WR_CLR_WRITE;
      _delay_us(1); //emprical
      WR_SET_IDLE;
      Text_Data++;
      }
    //Move to the next line in the text layer
    address+=BYTES_PER_LINE;
    //Move the pointer over the null terminator in the
    //flash array.
    Text_Data++;
    }
  //Deslect the chip
  CS_SET_DESELECT;
  }
//----------------------------------------------------------------------------
void Clear_Text_Layer(void)
  {
  uint8_t
    row;
  uint8_t
    col;
  uint16_t
    address;
  address=0;

  for(row=0;row<30;row++)
    {
    //Set the position, within the text layer
    write_command_S1D13700(0x46);
    write_data_S1D13700(address&0x00FF);
    write_data_S1D13700((address&0xFF00)>>8);
  
    //Clear loop
    // MEMWRITE command
    write_command_S1D13700(0x42);
    //Select the data register
    RS_CLR_DATA;
    //Select the chip
    CS_CLR_SELECT;

    //Everything gets a space
    LCD_DATA_SET_OUT=' ';

    for(col=0;col<40;col++)
      {
      //Make the write pulse
      _delay_us(1); //emprical
      WR_CLR_WRITE;
      _delay_us(1); //emprical
      WR_SET_IDLE;
      }
    address+=BYTES_PER_LINE;
    }
  //Deslect the chip
  CS_SET_DESELECT;
  }  
//----------------------------------------------------------------------------
#if GRAY == 0
// Feed this an array of bytes, each byte encoding 8 pixels, 1 bits per pixel.
void Show_Bitmap_Mono(const uint8_t *Bitmap_Data)
  {
  //Set the position, graphic layer
  write_command_S1D13700(0x46);
  write_data_S1D13700(0x60);
  write_data_S1D13700(0x09);

  write_command_S1D13700(0x42);

  uint8_t
    row;
  uint8_t
    col;

  //Select the data register
  RS_CLR_DATA;
  //Select the chip
  CS_CLR_SELECT;



  for(row=0;row<240;row++)
    {
    for(col=0;col<40;col++)
      {
      //Get the data from flash, send it to the controller.
      LCD_DATA_SET_OUT=pgm_read_byte(Bitmap_Data);
      //Make the write pulse
      _delay_us(1); //emprical
      WR_CLR_WRITE;
      _delay_us(1); //emprical
      WR_SET_IDLE;
      //Move the pointer to the next byte
      Bitmap_Data++;
      }
    }
  //Deslect the chip
  CS_SET_DESELECT;
  }
#endif
//============================================================================
#if GRAY
// Feed this an array of bytes, each byte encoding 4 pixels, 2 bits per pixel.
void Show_2_bit_Grey_Bitmap(const uint8_t *Bitmap_Data)
  {
  //Set the position, graphic layer
  write_command_S1D13700(0x46);
  write_data_S1D13700(0x60);
  write_data_S1D13700(0x09);

  write_command_S1D13700(0x42);

  uint8_t
    row;
  uint8_t
    col;

  //Select the data register
  RS_CLR_DATA;
  //Select the chip
  CS_CLR_SELECT;

  for(row=0;row<240;row++)
    {
    for(col=0;col<80;col++)
      {
      //Get the data from flash, send it to the controller.
      LCD_DATA_SET_OUT=pgm_read_byte(Bitmap_Data);
      //Make the write pulse
      _delay_us(1); //emprical
      WR_CLR_WRITE;
      _delay_us(1); //emprical
      WR_SET_IDLE;
      //Move the pointer to the next byte
      Bitmap_Data++;
      }
    }
  //Deslect the chip
  CS_SET_DESELECT;
  }
#endif //GRAY  
//============================================================================
void Fill_Graphic_Layer(uint8_t Fill_Data) 
  {
  //Set the position, graphic layer
  write_command_S1D13700(0x46);
  write_data_S1D13700(0x60);
  write_data_S1D13700(0x09);

  // MEMWRITE command
  write_command_S1D13700(0x42);

  uint8_t
    row;
  uint8_t
    col;

  //Select the data register
  RS_CLR_DATA;
  //Select the chip
  CS_CLR_SELECT;

  //Fill the display with the user's data
  LCD_DATA_SET_OUT=Fill_Data;
  
  for(row=0;row<240;row++)
    {
    for(col=0;col<BYTES_PER_LINE;col++)
      {
      //Make the write pulse
      _delay_us(1); //emprical
      WR_CLR_WRITE;
      _delay_us(1); //emprical
      WR_SET_IDLE;
      }
    }
  //Deslect the chip
  CS_SET_DESELECT;
  }
//============================================================================
void Horizontal_Stripes(void)
  {
  //Set the position, graphic layer
  write_command_S1D13700(0x46);
  write_data_S1D13700(0x60);
  write_data_S1D13700(0x09);

  // MEMWRITE command
  write_command_S1D13700(0x42);

  uint8_t
    row;
  uint8_t
    col;

  //Select the data register
  RS_CLR_DATA;
  //Select the chip
  CS_CLR_SELECT;
  
  for(row=0;row<240;row++)
    {
    //Two lines on, two lines off.
    if((row&0x03) < 0x02)
      {
      LCD_DATA_SET_OUT=0xFF;
      }
    else
      {
      LCD_DATA_SET_OUT=0x00;
      }
    for(col=0;col<BYTES_PER_LINE;col++)
      {
      //Make the write pulse
      _delay_us(1); //emprical
      WR_CLR_WRITE;
      _delay_us(1); //emprical
      WR_SET_IDLE;
      }
    }
  //Deslect the chip
  CS_SET_DESELECT;
  }
//============================================================================
void CheckerBoard(void)
  {
  //Set the position, graphic layer
  write_command_S1D13700(0x46);
  write_data_S1D13700(0x60);
  write_data_S1D13700(0x09);

  // MEMWRITE command
  write_command_S1D13700(0x42);

  uint8_t
    row;
  uint8_t
    col;

  //Select the data register
  RS_CLR_DATA;
  //Select the chip
  CS_CLR_SELECT;
  
  for(row=0;row<240;row++)
    {
#if GRAY
    //Two lines off, two lines even
    //Each byte has 4 pixels.
    if((row&0x03) < 0x02)
      {
      LCD_DATA_SET_OUT=0x0F;
      }
    else
      {
      LCD_DATA_SET_OUT=0xF0;
      }
#else
    //Two lines off, two lines even
    //Each byte has 8 pixels.
    if((row&0x03) < 0x02)
      {
      LCD_DATA_SET_OUT=0x33;
      }
    else
      {
      LCD_DATA_SET_OUT=0xCC;
      }
#endif
    for(col=0;col<BYTES_PER_LINE;col++)
      {
      //Make the write pulse
      _delay_us(1); //emprical
      WR_CLR_WRITE;
      _delay_us(1); //emprical
      WR_SET_IDLE;
      }
    }
  //Deslect the chip
  CS_SET_DESELECT;
  }
//============================================================================
void Vertical_Stripes(void)
  {
  //Set the position, graphic layer
  write_command_S1D13700(0x46);
  write_data_S1D13700(0x60);
  write_data_S1D13700(0x09);

  // MEMWRITE command
  write_command_S1D13700(0x42);

  uint8_t
    row;
  uint8_t
    col;

  //Select the data register
  RS_CLR_DATA;
  //Select the chip
  CS_CLR_SELECT;


#if GRAY
  //Each byte has 4 pixels.
  LCD_DATA_SET_OUT=0x0F;
#else
  //Each byte has 8 pixels.
  LCD_DATA_SET_OUT=0x33;
#endif

  for(row=0;row<240;row++)
    {
    for(col=0;col<BYTES_PER_LINE;col++)
      {
      //Make the write pulse
      _delay_us(1); //emprical
      WR_CLR_WRITE;
      _delay_us(1); //emprical
      WR_SET_IDLE;
      }
    }
  //Deslect the chip
  CS_SET_DESELECT;
  }
//============================================================================
void Initialize_CFAG320240CX()
  {
  //Clobber the display controller into submission. 
  RESET_CLR_RESET;
  _delay_ms(10);
  RESET_SET_OPERATE;
  _delay_ms(50);

  //SYSTEM_SET:
  write_command_S1D13700(0x40);

  write_data_S1D13700(0x30); //     REG[00h] Memory Configuration Register
  // 0011 0000  
  // |||| ||||-- bit 0: Character Generator Select (M0)
  // |||| |||      0=the internal CGROM is selected.
  // |||| |||      1=the internal CGRAM is selected.
  // |||| |||--- bit 1 Reserved = 0
  // |||| ||---- bit 2 Character Height (M2)
  // |||| |        When this bit = 0, the character height is 8 pixels.
  // |||| |        When this bit = 1, the character height is 16 pixels.
  // |||| |----- bit 3 Panel Drive Select (W/S)
  // ||||          When this bit = 0, a single panel drive is selected.
  // ||||          When this bit = 1, a dual panel drive is selected.
  // ||||
  // ||||------- bit 4 Reserved = 1
  // |||-------- bit 5 Screen Origin Compensation (IV)
  // ||            When this bit = 0, screen origin compensation is done.
  // ||            When this bit = 1, screen origin compensation is not done.
  // ||--------- bit 6 unused = 0
  // |---------- bit 7 unused = 0
  
  write_data_S1D13700(0x87); //FX=  REG[01h] Horizontal Character Size Register
  // 1000 0111  
  // |||| ||||-- Horizontal Character Size (FX) bits 0-3
  // ||||          These bits define the horizontal size, or width, of each character, in pixels.
  // ||||      REG[01h] bits 3-0 = Horizontal Character Size in pixels - 1
  // ||||------- bit 4,5,6 unused = 0
  // |---------- bit 7 MOD
  //       When this bit = 0, 16-line AC drive is selected.
  //       When this bit = 1, two-frame AC drive is selected.

  write_data_S1D13700(0x07); //FY=  REG[02h] Vertical Character Size Register
  // 0000 0111  
  // |||| ||||-- Vertical Character Size (FY) bits 0-3
  // ||||          These bits define the vertical size, or height, of each character, in pixels.
  // ||||      REG[02h] bits 3-0 = Vertical Character Size in pixels - 1
  // ||||------- bits 4-7 unused = 0

  write_data_S1D13700(BYTES_PER_LINE-1); //C/R= REG[03h] Character Bytes Per Row Register
  // These bits determine the size of each character row (or display line), in bytes,
  // to a maximum of 239. The value of these bits is defined in terms of C/R
  // REG[03h] bits 7-0 = ([C/R] x bpp) - 1  (bpp == bits per pixel)
  //
  //   C/R =(320 dots / 8 dots/character) = 40
  //   2 bpp = 80
  //   REG[03h] = [C/R] x bpp) - 1 = 79

  write_data_S1D13700(BYTES_PER_LINE+1); //TC/R=REG[04h] Total Character Bytes Per Row Register
  // These bits set the length of one line, including horizontal blanking, in bytes, 
  // to a maximum of 255. The value of these bits is defined in terms of TC/R TC/R 
  // can be adjusted to hold the frame period constant and minimize jitter for any 
  // given main oscillator frequency, fosc.
  // REG[04h] bits 7-0 = [TC/R] + 1
  // TC/R must be programmed such that the following formulas are valid.
  // [C/R] + 2 <= [TC/R]
  // 0 <= [TC/R] <= 255

  write_data_S1D13700(239); //L/F= REG[05h] Frame Height Register
  // These bits determine the frame height, in lines. The maximum frame height is 
  // 256 lines.
  // REG[05h] bits 7-0 = frame height in lines - 1.
  // 240 lines in this module, so REG[05h] = 240-1 = 239

#if GRAY
  write_data_S1D13700(0x50); //APL= REG[06h] Horizontal Address Range Register 0
#else
  write_data_S1D13700(0x28); //APL= REG[06h] Horizontal Address Range Register 0
#endif  
  write_data_S1D13700(0x00); //APH= REG[07h] Horizontal Address Range Register 1
  // These bits define the horizontal address range of the virtual screen. 
  // The maximum value for this register is 7FFFh.
  // REG[07h] bits 7-0, REG[06h] bits 7-0 = Addresses per line
  // Number of bytes used for one line = 80 = 0x0050
       
  //SCROLL:
  write_command_S1D13700(0x44);
  write_data_S1D13700(0x00); // REG[0Bh] Screen Block 1 Start Address Register 0
  write_data_S1D13700(0x00); // REG[0Ch] Screen Block 1 Start Address Register 1
  // Screen Block 1 Start Address (SAD1) bits [15:0]
  // These bits determine the memory start address of screen block 1.

  write_data_S1D13700(239); // REG[0Dh] Screen Block 1 Size Register
  // These bits determine the size of screen block 1, in lines.
  // REG[0Dh] bits 7-0 = screen block 1 size in number of lines - 1
  // 240 lines - 1 = 239

  write_data_S1D13700(0x60); // REG[0Eh] Screen Block 2 Start Address Register 0
  write_data_S1D13700(0x09); // REG[0Fh] Screen Block 2 Start Address Register 1
  // These bits determine the memory start address of screen block 2.
  // 0x0960

  write_data_S1D13700(239); // REG[10h] Screen Block 2 Size Register
  // These bits determine the size of screen block 2, in lines.
  // REG[10h] bits 7-0 = screen block 2 size in number of lines - 1
  // 240 lines - 1 = 239

  write_data_S1D13700(0x00); // REG[11h] Screen Block 3 Start Address Register 0
  write_data_S1D13700(0x00); // REG[12h] Screen Block 3 Start Address Register 1
  // These bits determine the memory start address of screen block 3.
  // 0x0000

  write_data_S1D13700(0x00); // REG[13h] Screen Block 4 Start Address Register 0
  write_data_S1D13700(0x00); // REG[14h] Screen Block 4 Start Address Register 1
  // These bits determine the memory start address of screen block 4.
  // 0x0000

  //HDOT_SCR:
  write_command_S1D13700(0x5A); // REG[1Bh] Horizontal Pixel Scroll Register
  write_data_S1D13700(0x00);  //NO SCROLL
  // 0000 0000  
  // |||| ||||-- Horizontal Pixel Scroll bit 1 - 3
  // |||| |----- bit 3-7 unused = 0
  // These bits specify the number of horizontal pixels to scroll the display. 
  // The character bytes per row (C/R), REG[03h] bits 7-0, must be set to one 
  // more than the actual number of horizontal characters before using horizontal 
  // pixel scroll. Smooth scrolling can be simulated by repeatedly changing the 
  // value of REG[1Bh] bits 2-0.

  //OVERLAY:  (Note: Screen blocks 2 and 4 can display graphics only.)
  write_command_S1D13700(0x5B); // REG[18h] Overlay Register
  
  write_data_S1D13700(0x01);   //GRAPHIC & XOR
  // 0000 0001  
  // |||| ||||-- bit 0 Layer Composition Method (MX) bit 1
  // |||| |||--- bit 1 Layer Composition Method (MX) bit 2
  // |||| ||       These bits select the layered screen composition method, 
  // |||| ||       which can be OR, AND, or Exclusive-OR.
  // |||| ||       
  // |||| ||---- bit 2 Screen Block 1 Display Mode (DM0)
  // |||| |        This bit determines the display mode for screen block 1.
  // |||| |    When this bit = 0, screen block 1 is configured for text mode.
  // |||| |        When this bit = 1, screen block 1 is configured for graphics mode.
  // |||| |
  // |||| |----- bit 3 Screen Block 3 Display Mode (DM1)
  // ||||          This bit determines the display mode for screen block 3.
  // ||||          When this bit = 0, screen block 3 is configured for text mode.
  // ||||          When this bit = 1, screen block 3 is configured for graphics mode.
  // ||||
  // ||||------- bit 4 3 Layer Overlay Select (OV)
  // |||           This bit determines how many layers are used when graphics mode is enabled. 
  // |||           - For mixed text and graphics, this bit must be set to 0.
  // |||           When this bit = 0, two layers are used.
  // |||           When this bit = 1, three layers are used.
  // |||
  // |||-------- bit 5-7 unused = 0

  //CSR_FORM:
  write_command_S1D13700(0x5D); 
  write_data_S1D13700(0x06); // REG[15h] Cursor Width Register
  // 0000 0110  
  // |||| ||||-- bit 0 Cursor Width (CRX) bit 0-3
  // ||||          These bits specify the width (or horizontal size) of the cursor, in pixels from 
  // ||||          the character origin 
  // ||||          REG[15h] bits 3-0 = cursor width in pixels - 1
  // ||||
  // ||||------- bit 4-7 unused = 0

  write_data_S1D13700(0x86); // REG[16h] Cursor Height Register
  // 1000 0110  
  // |||| ||||-- bit 0 Cursor Height (CRY) bit 0
  // |||| |||--- bit 1 Cursor Height (CRY) bit 1
  // |||| ||---- bit 2 Cursor Height (CRY) bit 2
  // |||| |----- bit 3 Cursor Height (CRY) bit 3
  // ||||          For an underscore cursor (REG[16h] bit 7 = 0), these bits set the location 
  // ||||          of the cursor, in lines from the character origin.
  // ||||          For a block cursor (REG[16h] bit 7 = 1), these bits set the height 
  // ||||          (or vertical size) of the cursor, in lines from the character origin.
  // ||||    REG[16h] bits 3-0 = cursor height in lines - 1
  // ||||
  // ||||------- bit 4-7 unused = 0
  // This bit determines the cursor mode. When graphics mode is selected, 
  // this bit must be set to 1.
  // When this bit = 0, an underscore cursor ( _ ) is selected.
  // When this bit = 1, a block cursor ( ¦ ) is selected.
  
  //CSR_DIR:
  write_command_S1D13700(0x4C); // REG[17h] Cursor Shift Direction Register  -- Default = 00h
  // 0100 11dd  
  // |||| ||||-- bit 1:0
  // |||| ||         0 0 Right (command 0x4C)
  // |||| ||         0 1 Left  (command 0x4D)
  // |||| ||         1 0 Up    (command 0x4E)
  // |||| ||         1 1 Down  (command 0x4F)
  // ||||-||---- bit 2-7 (command 0x4C base)

  //DISP_ON:
  write_command_S1D13700(0x59);
  write_data_S1D13700(0x14); // REG[0Ah] Display Attribute Register values
  // 0001 0000  
  // |||| ||||-- bit 0 Cursor Attribute (FC) bit 0
  // |||| |||--- bit 1 Cursor Attribute (FC) bit 1
  // |||| ||       These bits control the cursor and set the flash rate -- 70% duty cycle (70% on, 30% off):
  // |||| ||       0 0 OFF (Blank)
  // |||| ||       0 1 ON  No Flashing
  // |||| ||       1 0 ON  Flash at fFR/32 Hz (approx. 2 Hz)
  // |||| ||       1 1 ON  Flash at fFR/64 Hz (approx. 1 Hz)
  // |||| ||       NOTE:  When the cursor is disabled, a write to memory automatically enables the cursor and
  // |||| ||              places the cursor at the next memory location. A read from memory does not enable the
  // |||| ||              cursor, however, it still places the cursor at the next memory location.
  // |||| ||
  // |||| ||---- bit 2 SAD1 Attribute (FP 1-0) bit 0
  // |||| |----- bit 3 SAD1 Attribute (FP 1-0) bit 1
  // ||||          These bits control the attributes of the third screen block (SAD1):
  // ||||          0 0 OFF (Blank)
  // ||||          0 1 ON  No Flashing
  // ||||          1 0 ON  Flash at fFR/32 Hz (approx. 2 Hz)
  // ||||          1 1 ON  Flash at fFR/4 Hz (approx. 16 Hz)
  // ||||
  // ||||------- bit 4 SAD2 Attribute (FP 3-2) bit 0
  // |||-------- bit 5 SAD2 Attribute (FP 3-2) bit 1
  // ||            These bits control the attributes of the third screen block (SAD2):
  // ||            0 0 OFF (Blank)
  // ||            0 1 ON  No Flashing
  // ||            1 0 ON  Flash at fFR/32 Hz (approx. 2 Hz)
  // ||            1 1 ON  Flash at fFR/4 Hz (approx. 16 Hz)
  // ||            
  // ||--------- bit 6 SAD3 Attribute (FP 5-4) bit 0
  // |---------- bit 7 SAD3 Attribute (FP 5-4) bit 1
  //               These bits control the attributes of the third screen block (SAD3):
  //               0 0 OFF (Blank)
  //               0 1 ON  No Flashing
  //               1 0 ON  Flash at fFR/32 Hz (approx. 2 Hz)
  //               1 1 ON  Flash at fFR/4 Hz (approx. 16 Hz)

  //Grayscale
  write_command_S1D13700(0x60);
#if GRAY
  write_data_S1D13700(0x01); //REG[20h] Bit-Per-Pixel Select Register
#else
  write_data_S1D13700(0x00); //REG[20h] Bit-Per-Pixel Select Register
#endif  
  // 0000 11dd  
  // |||| ||||-- bit 1:0
  // |||| ||         0 0 Bits per pixel = 1
  // |||| ||         0 1 Bits per pixel = 2
  // |||| ||         1 0 Bits per pixel = 4
  // |||| ||         1 1 reserved
  // ||||-||---- bit 2-7 unused = 0


  Clear_Text_Layer();
  Fill_Graphic_Layer(0x00);
  }
//============================================================================
void setup()
  {
  //Initialize / idle the ports & start reset pulse.
  CS_SET_DESELECT;
  RESET_CLR_RESET;
  RS_SET_COMMAND;
  WR_SET_IDLE;
  RD_SET_IDLE;
  LCD_DATA_SET_OUT=0x00;
    
  //Set port directions
  DDRB = 0x03;
  DDRC = 0x07;
  SET_LCD_PORT_OUT;

  //Set up the S1D13700 controller
  Initialize_CFAG320240CX();
  }
//============================================================================
void loop()
  {
#if GRAY
  #if FRONT
    Show_2_bit_Grey_Bitmap(&Rover_Front_four_2_bit_pixels_per_byte[0][0]);
  #else //FRONT
    Show_2_bit_Grey_Bitmap(&Rover_Side_four_2_bit_pixels_per_byte[0][0]);
  #endif //FRONT
#else
  Show_Text_Layer(&Splash_Text_Screen[0][0]);
  Show_Bitmap_Mono(&splash_CFAG320240CX_320_240[0][0]);
#endif
  _delay_ms(3000);
  Clear_Text_Layer();
  Fill_Graphic_Layer(0x00);
  _delay_ms(3000);
  Fill_Graphic_Layer(0xFF);
  _delay_ms(3000);
  Horizontal_Stripes();
  _delay_ms(3000);
  CheckerBoard();
  _delay_ms(3000);
  Vertical_Stripes();
  _delay_ms(3000);

  Fill_Graphic_Layer(0x00);
  Show_Text_Layer(&Text_Demo_Screen[0][0]);
  _delay_ms(3000);
  Fill_Graphic_Layer(0xFF);
  _delay_ms(3000);
  Clear_Text_Layer();
  }
//============================================================================
